@@ -1,5 +1,5 @@
 package Mojo::SinaWeibo;
-$Mojo::SinaWeibo::VERSION = "1.1";
+$Mojo::SinaWeibo::VERSION = "1.2";
 use Mojo::Base 'Mojo::EventEmitter';
 use Mojo::JSON qw(encode_json decode_json);
 use Mojo::Util qw(b64_encode dumper sha1_sum url_escape url_unescape encode decode);
@@ -22,7 +22,7 @@ has log_level               => 'info';     #debug|info|warn|error|fatal
 has log_path                => undef;
 
 has max_timeout_count       => 3;
-has timeout                 => 8;
+has timeout                 => 10;
 has timeout_count           => 0;
 
 has log => sub{
@@ -147,10 +147,11 @@ sub auth {
     $s->prelogin();
     $s->login();
     if($s->login_state eq "success"){
+        $s->timeout_count(0);
         return $s
     }
-    $s->fatal("授权失败，程序退出");
-    exit;
+    $s->fatal("授权失败，程序处于离线状态");
+    $s->login_state("stop");
 }
 sub login {
     my $s = shift;
@@ -258,13 +259,19 @@ sub login {
             my $json = decode_json($1);
             return unless $json->{result};
             $s->uid($json->{userinfo}{uniqueid})->home("http://weibo.com/u/$json->{userinfo}{uniqueid}/home");
+            $s->debug("进行首页跳转...");
             if(defined $json->{redirect}){
-                $s->debug("进行首页跳转...");
                 my $tx = $s->ua->get($json->{redirect}) ;
                 return unless $tx->success;
                 $s->login_state("success");
                 $s->info("登录成功");
             }
+            else{
+                my $tx = $s->ua->get("http://weibo.com/" . $json->{userinfo}{userdomain});
+                return unless $tx->success;
+                $s->login_state("success");
+                $s->info("登录成功");
+            }
         }
     }
     elsif($retcode ==4049){
@@ -509,8 +516,15 @@ sub parse_im_msg{
         elsif($m->{channel} eq "/meta/subscribe"){
             return unless $m->{successful};
             $s->debug("收到服务器订阅响应消息");
-            $s->im_send($s->gen_im_msg("cmd",cmd=>"usersetting"));
-            $s->im_send($s->gen_im_msg("cmd",cmd=>"recents"));
+            if(@{$s->im_user} == 0){
+                $s->im_send($s->gen_im_msg("cmd",cmd=>"usersetting"));
+                $s->im_send($s->gen_im_msg("cmd",cmd=>"recents"));
+            }
+            else{
+                $s->im_ready(1);
+                $s->debug("私信服务器状态准备就绪");
+                $s->emit("im_ready");
+            }
         }
         elsif($m->{channel} eq "/im/req"){
             next unless $m->{successful};
@@ -519,9 +533,11 @@ sub parse_im_msg{
             return unless exists $m->{data}{type};
             if($m->{data}{type} eq "recents"){
                 $s->im_user([ map {{uid=>$_->[0],nick=>$_->[1]}} @{$m->{data}{recents}} ]);
-                $s->im_ready(1);
-                $s->debug("私信服务器状态准备就绪");
-                $s->emit("im_ready");
+                if(!$s->im_ready){
+                    $s->im_ready(1);
+                    $s->debug("私信服务器状态准备就绪");
+                    $s->emit("im_ready");
+                }
             }            
 
             elsif( $m->{data}{type} eq "msg"){
@@ -588,34 +604,22 @@ sub im_speek{
     my $content = shift;
     my $callback = pop;
     $content = decode("utf8",$content) if defined $content;
-
+    if($s->login_state eq "stop"){
+        $callback->(undef,{is_success=>0,code=>503,msg=>encode("utf8","响应超时")});
+        return;
+    }
     $s->auth() if $s->login_state eq "invalid";
     #timeout handle
-    my $id;
     my $cb = {
         cb=>sub{
-            Mojo::IOLoop->remove($id);
+            #Mojo::IOLoop->remove($id);
             $callback->(@_) if ref $callback eq "CODE"; 
         },
         status => 'wait',
-        msg=>undef,
     };
-    $id = $s->timer($s->timeout,sub{
-        $cb->{status} = 'abort';
-        $callback->(undef,{is_success=>0,code=>503,msg=>encode("utf8","响应超时")}) if ref $callback eq "CODE";
-        $s->warn("消息响应超时");
-        my $count = $s->timeout_count;
-        $s->timeout_count(++$count);
-        if($s->timeout_count >= $s->max_timeout_count){
-            $s->im_ready(0);
-            $s->login_state("invalid");
-            $s->emit("invalid");
-        }
-    }); 
-    #
+    $s->timer($s->timeout,sub{$s->emit(im_timeout=>$cb);});
     if($s->im_ready){
         my $msg = $s->gen_im_msg("cmd",cmd=>"msg",uid=>$uid,msg=>$content);
-        $cb->{msg} = $msg;
         $s->im_send($msg,$cb);
         
     }
@@ -624,7 +628,6 @@ sub im_speek{
             my $s = shift;
             return if $cb->{status} eq "abort";
             my $msg = $s->gen_im_msg("cmd",cmd=>"msg",uid=>$uid,msg=>$content);
-            $cb->{msg} = $msg;
             $s->im_send($msg,$cb);
         });
         $s->im_init();
@@ -650,9 +653,11 @@ sub im_send{
             my($msg,$status) = @_;
             if(defined $msg){
                 $msg->{nick} = encode("utf8",$msg->{nick});
+                $msg->{content} = encode("utf8",$msg->{content});
             }
             $status->{msg} = encode("utf8",$status->{msg});
             $cb->{cb}->($msg,$status);
+            $cb->{status} = "done";
         });
         #push @{$s->im_send_callback},$cb;
     };
@@ -676,7 +681,22 @@ sub im_send{
 }
 sub run{
     my $s = shift;
-    my %p = @_ if @_%2==0;
+    my %p = @_ if @_>1 and @_%2==0;
+    $s->on(im_timeout=>sub{
+        my $s = shift;
+        my $cb = shift;
+        return if $cb->{status} eq "done";
+        $s->warn("私信消息响应超时,放弃等待");
+        $cb->{status} = 'abort';
+        $cb->{cb}->(undef,{is_success=>0,code=>503,msg=>encode("utf8","响应超时")}) if ref $cb->{cb} eq "CODE";
+        my $count = $s->timeout_count;
+        $s->timeout_count(++$count);
+        if($s->timeout_count >= $s->max_timeout_count){
+            $s->im_ready(0);
+            $s->login_state("invalid");
+            $s->emit("invalid");
+        }
+    });
     $s->on(receive_message=>sub{
         my $s = shift;
         my $msg = shift;
@@ -696,7 +716,7 @@ sub run{
         $s->auth();
     });
 
-    if($p{enable_api_server} ==1){
+    if(exists $p{enable_api_server} and $p{enable_api_server} ==1){
         package Mojo::SinaWeibo::Openxiaoice;
         use Encode;
         use Mojolicious::Lite;
